#version 330 core

layout(location=0) out vec4 fragColor;

in vec3 P;
in vec3 N;
in vec2 T;
in vec4 S;

uniform sampler2D color_texture;
uniform vec3 lightDir;

uniform int pLightCount;
uniform vec3 pLightPos[12];
uniform int illuminatedFlag;
uniform vec3 fog_color;

in vec3 LightDirection_tangentspace;
in vec3 EyeDirection_tangentspace;
in vec3 TriangleNormalTangentSpace;

uniform mat4 DepthBiasMVP;
uniform int myTextureSampler;
uniform sampler2DShadow shadowMap;
uniform sampler2D normal_texture;

uniform bool disableNormalMapping;

void main()
{
	// Normal mapping
	float cosTheta;

	vec3 TextureNormal_tangentspace = normalize(texture(normal_texture, T).rgb * 2.0 - 1.0);
	vec3 l = normalize(LightDirection_tangentspace);
	if (disableNormalMapping){
		cosTheta = clamp( dot( N,normalize(lightDir-P) ), 0,1 );
	}else{
		cosTheta = clamp( dot( TextureNormal_tangentspace,l ), 0,1 );
	}

	vec3 E = normalize(EyeDirection_tangentspace);
	vec3 R = reflect(-lightDir,TextureNormal_tangentspace);
	float cosAlpha = clamp( dot( E,R ), 0,1 );

	// Color and light

	vec4 diffuseTexColor = vec4(texture(color_texture, T).rgb, 1.0);
	float visibility = texture( shadowMap, vec3(S.xy/S.w, (S.z)/S.w) );
	float shadowIntensity = 0.3;
	float smoothness = 100;
	float diffuseIntensity = 0.4 *max(visibility, shadowIntensity) * cosTheta;
	float specularIntensity = max(visibility, shadowIntensity) * pow(cosAlpha, smoothness);

	/*
	for (int i=0; i<pLightCount; i=i+1) {
			
		//Euclidic 3D distance
		float LP_distance = sqrt((P.x-pLightPos[i].x)*(P.x-pLightPos[i].x) 
								+(P.y-pLightPos[i].y)*(P.y-pLightPos[i].y) 
								+(P.z-pLightPos[i].z)*(P.z-pLightPos[i].z));
		float attenuation = 1;
		if (LP_distance != 0){attenuation = 1/LP_distance;}

		vec3 L = normalize(pLightPos[i]-P);
		diffuseIntensity += 3 * attenuation * attenuation * (max(intensity + max(dot(N, L), 0.0), 1.0));
	}*/

	const float LOG2 = 1.442695;
	float z = gl_FragCoord.z / gl_FragCoord.w /200;
	float density = 2;
	float depthFogFactor = exp2( -density * 
						density * 
						z * 
						z * 
						LOG2 );
	float groundProximityFogFactor = clamp(1-P.y/50, 0.0, 1.0) *2;
	float fogFactor = clamp(groundProximityFogFactor * clamp(1-depthFogFactor, 0.0, 1.0), 0.0, 1.0 );
	vec3 finalColor = vec3(diffuseIntensity * diffuseTexColor.r + specularIntensity, diffuseIntensity * diffuseTexColor.g + specularIntensity, diffuseIntensity * diffuseTexColor.b + specularIntensity);

	fragColor = vec4(
		fogFactor*fog_color.r + (1-fogFactor)*finalColor.r,
		fogFactor*fog_color.g + (1-fogFactor)*finalColor.g,
		fogFactor*fog_color.b + (1-fogFactor)*finalColor.b,
		1.0
	);

	//Light Position Test:
	//fragColor= vec4(pLightPos[0].x, pLightPos[0].y, pLightPos[0].z, 1.0);

	//Position Test:
	//fragColor = vec4(P.x/10, P.z/10, 0.0, 1.0);
}