#version 330

layout(location=0) in vec3 position;
layout(location=1) in vec3 normal;
layout(location=2) in vec2 uv;
layout(location=3) in vec3 tangents;
layout(location=4) in vec3 bitangents;

out vec3 P;
out vec3 N;
out vec2 T;
out vec4 S;
out vec3 MainLightDirection_tangentspace;
out vec3 EyeDirection_tangentspace;
out vec3 pLightDirection_tangentspace[12];

uniform mat4 view_proj;
uniform mat4 model;
uniform mat4 biasVP;
uniform mat4 view;

uniform vec3 mainLight;
uniform vec3 pLightPos[12];
uniform int pLightCount;

void main()
{
	S = biasVP * model * vec4(position,1);
	P = (model * vec4(position, 1)).xyz;
	N = normalize((model * vec4(normal, 0)).xyz);
	T = uv;

	// Normal mapping

	// TBN
	mat3 MV3x3 = mat3(view * model);
	vec3 vertexNormal_cameraspace = MV3x3 * normalize(normal);
    vec3 vertexTangent_cameraspace = MV3x3 * normalize(tangents);
    vec3 vertexBitangent_cameraspace = MV3x3 * normalize(bitangents);
	mat3 TBN = transpose(mat3(
		vertexTangent_cameraspace,
		vertexBitangent_cameraspace,
		vertexNormal_cameraspace
	));

	// Eye
	vec3 vertexPosition_cameraspace = ( view * model * vec4(position,1)).xyz;
	vec3 EyeDirection_cameraspace = vec3(0,0,0) - vertexPosition_cameraspace;

	// Light
	vec3 MainLightPosition_cameraspace = ( view * vec4(mainLight, 1)).xyz;
	vec3 MainLightDirection_cameraspace = MainLightPosition_cameraspace + EyeDirection_cameraspace;

	// Light and Eye
	MainLightDirection_tangentspace = TBN * MainLightDirection_cameraspace;

	for (int i=0; i<pLightCount; i++){
		vec3 pLightPosition_cameraspace = ( view * vec4(pLightPos[i], 1)).xyz;
		vec3 pLightDirection_cameraspace = pLightPosition_cameraspace + EyeDirection_cameraspace;
		pLightDirection_tangentspace[i] = TBN * pLightDirection_cameraspace;
	}

	MainLightDirection_tangentspace = TBN * MainLightDirection_cameraspace;
	EyeDirection_tangentspace =  TBN * EyeDirection_cameraspace;
	
	gl_Position = view_proj * model * vec4(position,1);
}