#version 330 core

// Interpolated values from the vertex shaders
in vec2 UV;
in vec4 myPosition;
in vec4 myPosition_world;

// Ouput data
out vec4 color;

uniform sampler2D myTextureSampler;
uniform sampler2D depthTexture;
uniform vec3 cameraPos;
uniform bool allowLensFlare;
uniform bool disableSoftParticles;

void main(){
	vec2 textureAccessCoords = vec2(0.5*((myPosition.x /myPosition.w) +1), 0.5*((myPosition.y /myPosition.w)+1)) ;
	vec4 objectsDepth = texture(depthTexture, textureAccessCoords);

	float objFalloffScale = 5.0;
	float objContrastPower = 2.0;
	float objDist = distance(myPosition_world.xyz, objectsDepth.xyz) / objFalloffScale;
	float objOutputAlpha = clamp(0.5*pow(objDist,objContrastPower),0,1);  

	float camFalloffScale = 3.0;
	float camContrastPower = 1.6;
	float camDist = distance(cameraPos, myPosition_world.xyz) / camFalloffScale;
	float camOutputAlpha = clamp(0.5*pow(camDist,camContrastPower),0,1);  
	
	if (disableSoftParticles){
		
	}

	// Output color = color of the texture at the specified UV
	vec4 texColor = texture2D( myTextureSampler, UV );
	
	color = vec4(min(texColor.rgb,allowLensFlare?1.0:0.9), texColor.a * (disableSoftParticles?1:(objOutputAlpha * camOutputAlpha)));

}
