#version 330 core

layout(location=0) out vec4 fragColor;

in vec3 P;
in vec3 N;
in vec2 T;

uniform sampler2D color_texture;
uniform vec3 fog_color;

void main()
{
	vec4 diffuseTexColor = vec4(texture(color_texture, T).rgb, 1.0);

	const float LOG2 = 1.442695;
	float z = gl_FragCoord.z / gl_FragCoord.w /200;
	float density = 0.7;
	float depthFogFactor = exp2( -density * 
						density * 
						z * 
						z * 
						LOG2 );
	float groundProximityFogFactor = clamp(pow(1-min(P.y/200,1), 2), 0.0, 1.0)*2;
	float fogFactor = clamp(groundProximityFogFactor * clamp(1-depthFogFactor, 0.0, 1.0), 0.0, 1.0 );

	fragColor = vec4(
		fogFactor*fog_color.r + (1-fogFactor)*diffuseTexColor.r,
		fogFactor*fog_color.g + (1-fogFactor)*diffuseTexColor.g,
		fogFactor*fog_color.b + (1-fogFactor)*diffuseTexColor.b,
		1.0
	);
}